@@ -107,7 +107,7 @@ collapse_slash_runs (char *path, int from_offset)
 	}
 
 	if (from_offset < i) {
-		strcpy (path + from_offset, path + i);
+		memmove (path + from_offset, path + i, strlen (path + i) + 1);
 		i = from_offset + 1;
 	}
 }
@@ -149,7 +149,8 @@ _gnome_vfs_canonicalize_pathname (gchar *path)
 
 			/* Handle `./'. */
 			if (path[i + 1] == GNOME_VFS_URI_PATH_CHR) {
-				strcpy (path + i, path + i + 2);
+				memmove (path + i, path + i + 2, 
+					 strlen (path + i + 2) + 1);
 				if (i == 0) {
 					/* don't leave leading '/' for paths that started
 					 * as relative (.//foo)
@@ -186,7 +187,8 @@ _gnome_vfs_canonicalize_pathname (gchar *path)
 						i++;
 					}
 
-					strcpy (path + marker, path + i + 2);
+					memmove (path + marker, path + i + 2,
+						 strlen (path + i + 2) + 1);
 					i = marker;
 				} else {
 					i = 2;
@@ -288,7 +290,7 @@ gnome_vfs_forkexec (const gchar *file_name,
  * 
  * Return value: 
  **/
-GnomeVFSProcessResult
+GnomeVFSProcessRunResult
 gnome_vfs_process_run_cancellable (const gchar *file_name,
 				   const gchar * const argv[],
 				   GnomeVFSProcessOptions options,
@@ -653,6 +655,7 @@ _gnome_vfs_url_show_using_handler_with_env (const char  *url,
 	if (template == NULL) {
 		g_free (template);
 		g_free (scheme);
+		g_object_unref (G_OBJECT (client));
 		return GNOME_VFS_ERROR_NO_HANDLER;
 	}
 	
@@ -662,6 +665,7 @@ _gnome_vfs_url_show_using_handler_with_env (const char  *url,
 				 NULL)) {
 		g_free (template);
 		g_free (scheme);
+		g_object_unref (G_OBJECT (client));
 		return GNOME_VFS_ERROR_PARSE;
 	}
 
