@@ -94,17 +94,19 @@ typedef struct {
 } UriStrspnSet; 
 
 UriStrspnSet uri_strspn_sets[] = {
-	{":@" GNOME_VFS_URI_PATH_STR, FALSE, ""},
+	{":@]" GNOME_VFS_URI_PATH_STR, FALSE, ""},
 	{"@" GNOME_VFS_URI_PATH_STR, FALSE, ""},
-	{":" GNOME_VFS_URI_PATH_STR, FALSE, ""}
+	{":" GNOME_VFS_URI_PATH_STR, FALSE, ""},
+	{"]" GNOME_VFS_URI_PATH_STR, FALSE, ""}
 };
 
 #define URI_DELIMITER_ALL_SET (uri_strspn_sets + 0)
 #define URI_DELIMITER_USER_SET (uri_strspn_sets + 1)
 #define URI_DELIMITER_HOST_SET (uri_strspn_sets + 2)
+#define URI_DELIMITER_IPV6_SET (uri_strspn_sets + 3)
 
 #define BV_SET(bv, idx) (bv)[((guchar)(idx))>>3] |= (1 << ( (idx) & 7) )
-#define BV_IS_SET(bv, idx) ((bv)[(idx)>>3] & (1 << ( (idx) & 7)))
+#define BV_IS_SET(bv, idx) ((bv)[((guchar)(idx))>>3] & (1 << ( (idx) & 7)))
 
 static const char *
 uri_strspn_to(const char *str, UriStrspnSet *set, const char *path_end)
@@ -170,6 +172,35 @@ split_toplevel_uri (const gchar *path, guint path_len,
 	cur_tok_start = path;
 	cur = uri_strspn_to (cur_tok_start, URI_DELIMITER_ALL_SET, path_end);
 
+	if (cur != NULL) {
+		const char *tmp;
+
+		if (*cur == ':') {
+			/* This ':' belongs to username or IPv6 address.*/
+			tmp = uri_strspn_to (cur_tok_start, URI_DELIMITER_USER_SET, path_end);
+
+			if (tmp == NULL || *tmp != '@') {
+				tmp = uri_strspn_to (cur_tok_start, URI_DELIMITER_IPV6_SET, path_end);
+
+				if (tmp != NULL && *tmp == ']') {
+					cur = tmp;
+				}
+			}
+		}
+	}
+
+	if (cur != NULL) {
+
+		/* Check for IPv6 address. */
+		if (*cur == ']') {
+
+			/*  No username:password in the URI  */
+			/*  cur points to ']'  */
+
+			cur = uri_strspn_to (cur, URI_DELIMITER_HOST_SET, path_end);
+		}
+	}
+
 	if (cur != NULL) {
 		next_delimiter = uri_strspn_to (cur, URI_DELIMITER_USER_SET, path_end);
 	} else {
@@ -201,7 +232,15 @@ split_toplevel_uri (const gchar *path, guint path_len,
 
 		if (*cur != '/') {
 			URI_MOVE_PAST_DELIMITER;
-			cur = uri_strspn_to (cur_tok_start, URI_DELIMITER_HOST_SET, path_end);
+
+			/* Move cur to point to ':' after ']' */
+			cur = uri_strspn_to (cur_tok_start, URI_DELIMITER_IPV6_SET, path_end);
+
+			if (cur != NULL && *cur == ']') {  /* For IPv6 address */
+				cur = uri_strspn_to (cur, URI_DELIMITER_HOST_SET, path_end);
+			} else {
+				cur = uri_strspn_to (cur_tok_start, URI_DELIMITER_HOST_SET, path_end);
+			}
 		} else {
 			cur_tok_start = cur;
 		}
@@ -278,7 +317,16 @@ split_toplevel_uri (const gchar *path, guint path_len,
 
 done:
 	if (*host_return != NULL) {
-		host = g_ascii_strdown (*host_return, -1);
+
+		/* Check for an IPv6 address in square brackets.*/
+		if (strchr (*host_return, '[') && strchr (*host_return, ']') && strchr (*host_return, ':')) {
+
+			/* Extract the IPv6 address from square braced string. */
+			host = g_ascii_strdown ((*host_return) + 1, strlen (*host_return) - 2);
+		} else {
+			host = g_ascii_strdown (*host_return, -1);
+		}
+
 		g_free (*host_return);
 		*host_return = host;
 
@@ -1095,7 +1143,16 @@ gnome_vfs_uri_to_string (const GnomeVFSURI *uri,
 
 		if (top_level_uri->host_name != NULL
 			&& (hide_options & GNOME_VFS_URI_HIDE_HOST_NAME) == 0) {
-		       	g_string_append (string, top_level_uri->host_name);
+
+			/* Check for an IPv6 address. */
+
+			if (strchr (top_level_uri->host_name, ':')) {
+				g_string_append_c (string, '[');
+				g_string_append (string, top_level_uri->host_name);
+				g_string_append_c (string, ']');
+			} else {
+				g_string_append (string, top_level_uri->host_name);
+			}
 		}
 		
 		if (top_level_uri->host_port > 0 
@@ -1118,9 +1175,11 @@ gnome_vfs_uri_to_string (const GnomeVFSURI *uri,
 	}
 
 	if (uri->parent != NULL) {
+		gchar *uri_str;
+		uri_str = gnome_vfs_uri_to_string (uri->parent, hide_options);
 		g_string_prepend_c (string, '#');
-		g_string_prepend (string, gnome_vfs_uri_to_string (uri->parent, 
-								   hide_options));
+		g_string_prepend (string, uri_str);
+		g_free (uri_str);
 	}
 
 	result = string->str;
@@ -1890,7 +1949,8 @@ gnome_vfs_uri_list_free (GList *list)
  * Returns a full URI given a full base URI, and a secondary URI which may
  * be relative.
  *
- * Return value: the URI (NULL for some bad errors).
+ * Return value: a newly allocated string containing the URI 
+ * (NULL for some bad errors).
  **/
 char *
 gnome_vfs_uri_make_full_from_relative (const char *base_uri,
@@ -1980,3 +2040,27 @@ gnome_vfs_uri_list_parse (const gchar* uri_list)
 
 	return g_list_reverse (result);
 }
+
+/* enumerations from "gnome-vfs-uri.h"
+ * This is normally automatically-generated, but glib-mkenums could not
+ * guess the correct get_type() function name for this enum.
+ */
+GType
+gnome_vfs_uri_hide_options_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GFlagsValue values[] = {
+      { GNOME_VFS_URI_HIDE_NONE, "GNOME_VFS_URI_HIDE_NONE", "none" },
+      { GNOME_VFS_URI_HIDE_USER_NAME, "GNOME_VFS_URI_HIDE_USER_NAME", "user-name" },
+      { GNOME_VFS_URI_HIDE_PASSWORD, "GNOME_VFS_URI_HIDE_PASSWORD", "password" },
+      { GNOME_VFS_URI_HIDE_HOST_NAME, "GNOME_VFS_URI_HIDE_HOST_NAME", "host-name" },
+      { GNOME_VFS_URI_HIDE_HOST_PORT, "GNOME_VFS_URI_HIDE_HOST_PORT", "host-port" },
+      { GNOME_VFS_URI_HIDE_TOPLEVEL_METHOD, "GNOME_VFS_URI_HIDE_TOPLEVEL_METHOD", "toplevel-method" },
+      { GNOME_VFS_URI_HIDE_FRAGMENT_IDENTIFIER, "GNOME_VFS_URI_HIDE_FRAGMENT_IDENTIFIER", "fragment-identifier" },
+      { 0, NULL, NULL }
+    };
+    etype = g_flags_register_static ("GnomeVFSURIHideOptions", values);
+  }
+  return etype;
+}
