@@ -161,15 +161,17 @@ force_probe (void)
 	} else {
 		client = _gnome_vfs_get_client ();
 		daemon = _gnome_vfs_client_get_daemon (client);
-		
-		CORBA_exception_init (&ev);
-		GNOME_VFS_Daemon_forceProbe (daemon,
-					     BONOBO_OBJREF (client),
-					     &ev);
-		if (BONOBO_EX (&ev)) {
-			CORBA_exception_free (&ev);
+
+		if (daemon != CORBA_OBJECT_NIL) {
+			CORBA_exception_init (&ev);
+			GNOME_VFS_Daemon_forceProbe (daemon,
+						     BONOBO_OBJREF (client),
+						     &ev);
+			if (BONOBO_EX (&ev)) {
+				CORBA_exception_free (&ev);
+			}
+			CORBA_Object_release (daemon, NULL);
 		}
-		CORBA_Object_release (daemon, NULL);
 	}
 }
 
@@ -257,12 +259,23 @@ mount_unmount_thread (void *arg)
 	}
 
 	if (info->should_eject) {
+#ifdef __FreeBSD__
+	    	char *argv[5] = {
+		    	"cdcontrol",
+			"-f",
+			NULL,
+			"eject",
+			NULL
+		};
+		argv[2] = info->device_path?info->device_path:info->mount_point;
+#else
 		char *argv[3] = {
 			"eject",
 			NULL,
 			NULL
 		};
 		argv[1] = info->device_path?info->device_path:info->mount_point;
+#endif
 
 		if (g_spawn_sync (NULL,
 				  argv,
@@ -377,15 +390,25 @@ emit_pre_unmount (GnomeVFSVolume *volume)
 		client = _gnome_vfs_get_client ();
 		daemon = _gnome_vfs_client_get_daemon (client);
 		
-		CORBA_exception_init (&ev);
-		 GNOME_VFS_Daemon_emitPreUnmountVolume (daemon,
-							BONOBO_OBJREF (client),
-							gnome_vfs_volume_get_id (volume),
-							&ev);
-		 if (BONOBO_EX (&ev)) {
-			 CORBA_exception_free (&ev);
-		 }
-		 CORBA_Object_release (daemon, NULL);
+		if (daemon != CORBA_OBJECT_NIL) {
+			CORBA_exception_init (&ev);
+			GNOME_VFS_Daemon_emitPreUnmountVolume (daemon,
+							       BONOBO_OBJREF (client),
+							       gnome_vfs_volume_get_id (volume),
+							       &ev);
+			if (BONOBO_EX (&ev)) {
+				CORBA_exception_free (&ev);
+			}
+			CORBA_Object_release (daemon, NULL);
+		}
+		/* Do a synchronous pre_unmount for this client too, avoiding
+		 * races at least in this process
+		 */
+		gnome_vfs_volume_monitor_emit_pre_unmount (volume_monitor,
+							   volume);
+		/* sleep for a while to get other apps to release their
+		 * hold on the device */
+		g_usleep (0.5*G_USEC_PER_SEC);
 	}
 }
 
